/////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	Moises Melendez, sesowm@yahoo.com, julio 2016
//
//	Aplicacion ESP8266-BOLT que conecta el modulo ESP8266 a una red wi-fi usando el sistema BOLT.
//
//	El programa se comunica con el mdulo ESP8266 mediante el puerto serie a 115200 bauds, esta es la 
//	velocidad predeterminada para la comunicacin con el mdulo, se pude cambiar mediante comandos AT
//	si se requiere de otra velocidad de funcionamiento.
//
//	Las conexiones son como se muestran:
//
//					  ____________________________
//					 |  						  |
//	 ________________|__________		 _________|__________________		 _______________________
//	|				 |		    |		| 	   	PIN 9				 |	    |DB9  SISTEMA BOLT
//	|			  RX    VCC---+-------|3.3V					PIN 8|------| 2	   TX_PIC	
//	|ESP8266  GPIO 0    RESET |	  	|   	  MAX3232 y		PIN 7|------| 3	   RX_PIC
//	|		  GPIO 2    CH_PD-+-------|3.3V	REGULADOR 3.3V    GND|------| 5	   GND
//	|			 GND    TX    |		|			    		  +5V|------| 7	   +5V
//	|________________|__|_______|		|__PIN 10____GND_____________|		|
//					 |  |					 |  	  |						|
//					 |  |____________________|		  |						|
//					 |								  |
//					 |________________________________|						
//
//
//		Nota: Los numero de pin dentro del bloque MAX3232 y REGULADOR 3.3V corresponden al nmero
//	de pin del circuito integrado MAX3232, este circuito integrado debe alimentarse con 3.3 volts, para 
//	obtener los 3.3 volts a partir de la alimentacin de 5 volts del sistema Bolt se debe utilIzar un 
// 	circuito integrado regulador de voltaje, por ejemplo AMS1117-3.3 o KA78R33
//
// 	Mientras se realiza el proceso de crear el servidor TCP/IP, se muestra en la pantalla LCD el efecto 
//	de cada uno de los comandos AT enviados al mdulo ESP8266, si se logra conectar el mdulo a la red
//	wi-fi se muesta en la pantalla LCD del sistema Bolt que la conexion fue exitosa.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////
//																									//
#define RED		"\"INFINITUMB001\""		//"ESCRIBA EL NOMBRE DE LA RED"										//
#define CLAVE 	"\"7409999507\""		//"ESCRIBA LA CLAVE DE LA RED"											//
//																									//
//////////////////////////////////////////////////////////////////////////////////////////////////////

#include <p18cxxx.h>
#include <delays.h>
#include <usart.h>
#include <string.h>
#include <stdio.h>		//libreria sprintf()
#include <stdlib.h>		//librerias atoi(),atof()
#include "18F2550BOLT.h"
#include "PUERTO-SERIAL-BOLT.h"
#include "ONEWIRE-BOLT.h"

#define CONV const far rom char *
#define INCOMING_SIZE 235		// Tamao del arreglo de datos recibidos
#define OK			0
#define ERROR		1
#define NO_ANSWER	2
#define SLEEP		while(1){}

// Prototipo de funciones
void rx_handler (void);											// Prototipo de ISR USART
void initConfiguration(void);									// Configuraciones iniciales
void writeLCDmsg(const far rom char * message);					// Escribe una cadena en el LCD
unsigned char sendATcommand(const far rom char * command, const far rom char * answer, unsigned int delay, unsigned int timeout);
																// Envia los comandos AT al modulo
void sendSTATUS2LCD(unsigned char STATUS, const far rom char * message2LCD);	// Escribe el estado del proceso de conexion
void sendATdata(const far rom char *data);						// Envia datos al modulo 
unsigned char conect2WI_FI(void);								// Funcion que conecta el modulo a una red wi-fi
void noConection(void);											// No es posible establecer la conexion

// Variables globales
volatile static char serialIncomingData[INCOMING_SIZE];			// Datos provenientes del puerto serie
volatile static unsigned char INDEX=0, INTERRUPT=0;				// Variables para el control de la interrupcion

extern void _startup( void ); // See c018i.c in your C18 compiler dir 
#pragma code _RESET_INTERRUPT_VECTOR = 0x000800 		// Vector de reset
void _reset( void ) 
{ 
    _asm goto _startup _endasm 
} 

#pragma code rx_interrupt = 0x000808					// Vector de interrupcion de alta prioridad
void rx_int (void)
{
	_asm goto rx_handler _endasm
}

#pragma code

#pragma interrupt rx_handler	// Funcion para el manejo de la interrupcion 
void rx_handler (void)
{
	serialIncomingData[INDEX++] = ReadUSART();	// Lee el buffer de entrada del puerto seria del pic
	PIR1bits.RCIF = 0;							// Limpia la bandera de interrupcion
	INTERRUPT=1;								// La interrupcion ha ocurrido
}
 
void main()						// Funcion principal
{
	unsigned char STATUS;		// Variable de control para monitorear el estado de la conexion
	initConfiguration();		// Configuracines iniciales 
	STATUS=conect2WI_FI();		// 
	if(STATUS==OK)				// Si la conexion se ha establecido
		SLEEP
	else						// Si la conexion no se ha establecido 
		noConection();			// Envia un mensaje de error
} 

void noConection(void)						// Si la conexion no se ha establecido
{											// Envia un mensaje de error a traves del LCD
	writeLCDmsg((CONV)"  No se puede   ");	// del sistema BOLT
	delay_ms(800);
	writeLCDmsg((CONV)" conectar a la  ");
	delay_ms(800);
	writeLCDmsg((CONV)"   red wi-fi    ");
	delay_ms(800);
	writeLCDmsg((CONV)"  Sin conexion  ");
	SLEEP;									// Loop infinito.
}

//
//	Funcion que envia todos los comandos AT necesarion para establecer la comunicacion con un 
//	servidor remoto a traves de el modulo sim900, se requiere de conocer la IP y el 
//	numero de puerto del servidor al que se requiere conectar. Ademas monitorea constantemente
//	el estado de la conexion, si no es posible establecerla debido a que alguno de los comandos
//	no puede procesarse la funcion retorna un valor de ERROR.
//
unsigned char conect2WI_FI(void)
{
	unsigned char STATUS;
	writeLCDmsg((CONV)"  BOLT-MSP8266  ");
	delay_ms(1000);
	writeLCDmsg((CONV)"   Aplicacion   ");
	delay_ms(1000);
	writeLCDmsg((CONV)" Conexion wi-fi ");
	delay_ms(1000);

	STATUS=sendATcommand((CONV)"AT",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"       AT       ");
	if(STATUS!=OK) return ERROR;

	STATUS=sendATcommand((CONV)"AT+CWQAP",(CONV)"OK",5,2);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CWQAP     ");
	if(STATUS!=OK) return ERROR;

	STATUS=sendATcommand((CONV)"AT+CWMODE=3",(CONV)"OK",2,2);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CWMODE    ");
	if(STATUS!=OK) return ERROR;

	STATUS=sendATcommand((CONV)"AT+CWJAP="RED","CLAVE,(CONV)"WIFI CONNECTED",12,2);
	//STATUS=sendATcommand((CONV)"AT+CWJAP=\"MONKEY\",\"rt_34Le12tq89\"",(CONV)"WIFI CONNECTED",12,2);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CWJAP    ");
	if(STATUS!=OK) return ERROR;

	STATUS=sendATcommand((CONV)"AT+CWJAP?",(CONV)"OK",2,2);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CWJAP?    ");
	if(STATUS!=OK) return ERROR;

	STATUS=sendATcommand((CONV)"AT+CIFSR",(CONV)"OK",2,2);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CIFSR     ");
	if(STATUS!=OK) return ERROR;


	writeLCDmsg((CONV)"Conectado a red ");
	delay_ms(1000);
	writeLCDmsg((CONV)RED"           ");
	delay_ms(2000);
	return OK;		
}

void initConfiguration(void)	// Configuraciones iniciales para el sistema Bolt, cofigura el LCD, los puertos, el relevador
{								// y el puerto serie en hardware
	int initIndex;			// Variable local para limpiar el arreglo que contendra los datos recibidos
	init_bolt();			
	InitLCD();
	
	OpenUSART(USART_TX_INT_OFF & USART_RX_INT_ON &
	USART_ASYNCH_MODE & USART_EIGHT_BIT &
	USART_CONT_RX & USART_BRGH_HIGH, 25);
		
	RCONbits.IPEN = 1;		// Habilita prioridad en las interrupciones
	IPR1bits.RCIP = 1;		// Alta prioridad para la interrupcion por llegada de datos
	INTCONbits.GIEH = 1;	// Habilita todas las interrupciones de alta prioridad
	for(initIndex=0;initIndex<INCOMING_SIZE;initIndex++) serialIncomingData[initIndex]=0;
}

void sendATdata(const far rom char *data)	// Envia los datos al modulo 
{
	int i;
	for(i=0; i<strlenpgm(data); i++) 
	{
		while(BusyUSART());				// Espera a que el puerto serie este desocupado
		putcUSART(data[i]);				// Escribe cada uno de los caracteres al puerto serie
	}
	while(BusyUSART());
	putcUSART('\r');					// Retorno de carro
	while(BusyUSART());
	putcUSART((char)10);
	delay_ms(200);
}

void writeLCDmsg(const far rom char * message)		// Escribe la cadena "message" al LCD
{
	unsigned char i;
	ClearScreen();							// Limpia la pantalla
	GotoXY(0,0);							// Ve a la posicion (0,0)
	for(i=0; i<8; i++) PutLCD(message[i]);	// Imprime cada uno de los caracteres de la cadena del argumento
	GotoXY(1,0);
	for(i=8; i<strlenpgm(message);i++) PutLCD(message[i]);
}

//	
//	Funcion que envia un comando AT al modulo, recibe el comando AT, la respuesta esperada, el retardo para el
//	procesamiento del comando AT por parte del modulo, y un tiempo limite, si el modulo no responde en 
//	el lapso de tiempo indicado por estr numero la funcion retorna el valor NO_ANSWER
//
unsigned char sendATcommand(const far rom char * command, const far rom char * answer, unsigned int delay, unsigned int timeout)
{
	unsigned long int time;
	int i;
	for(i=0; i<strlenpgm(command); i++) 
	{
		while(BusyUSART());		// Espera a que este disponible el modulo USART
		putcUSART(command[i]);	// envia cada uno de los caracteres del comando AT
	}
	while(BusyUSART());
	putcUSART('\r');			// equivale a pulsar ENTER
	while(BusyUSART());
	putcUSART((char)10);		
	delay_ms(delay*1000);
	for(time=0;time<(timeout*200000);time++)
	{
		if(INTERRUPT)
		{
			if(strstrrampgm(serialIncomingData,answer)) 
			{	
				INDEX=0;
				INTERRUPT=0;
				for(i=0;i<INCOMING_SIZE;i++) serialIncomingData[i]=0;
				return OK;
			}
			else 
			{	
				INDEX=0;
				INTERRUPT=0;
				for(i=0;i<INCOMING_SIZE;i++) serialIncomingData[i]=0;
				return ERROR;
			}
		}
		else continue;
	}
	INDEX=0;
	INTERRUPT=0;
	for(i=0;i<INCOMING_SIZE;i++) serialIncomingData[i]=0;
	return NO_ANSWER;
}

//
//	Funcion que envia un mensaje en el LCD indicando si un comando AT ha sido procesado correctamente
//
void sendSTATUS2LCD(unsigned char STATUS, const far rom char * message2LCD)
{
	writeLCDmsg(message2LCD);
	delay_ms(500);
	switch(STATUS){
	case OK:{ 
		writeLCDmsg((CONV)"       OK       ");
		delay_ms(500);
		break;}
	case ERROR:{
		writeLCDmsg((CONV)"      Error     ");
		delay_ms(500);
		break;}
	case NO_ANSWER:{
		writeLCDmsg((CONV)" Sin respuesta  ");
		delay_ms(500);
		break;}
	default: 
		break;}
} 


